package sentimentanalyzer;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import opennlp.tools.doccat.DoccatModel;
import opennlp.tools.doccat.DocumentCategorizerME;

/**
 * Třída pro analýzu sentimentu za použití vytvořeného modelu
 * 
 * @author Petr Škola
 */
public class Analyzer {
	private DocumentCategorizerME myCategorizer;
	
  /**
   * 
   */
	public Analyzer(){
		InputStream modelIn = null;
		DoccatModel m = null;
		try {
			modelIn = new FileInputStream("model/model.bin");
			m = new DoccatModel(modelIn);
		} catch (FileNotFoundException ex) {
			Logger.getLogger(Analyzer.class.getName()).log(Level.SEVERE, null, ex);
		} catch (IOException ex) {
			Logger.getLogger(Analyzer.class.getName()).log(Level.SEVERE, null, ex);
		}
		this.myCategorizer = new DocumentCategorizerME(m);
	}
	
	public SentimentResult detectSentiment(String post) {
		//String[] posts = new String[1];
		//posts[0] = post;
		if (post == null || post.length()==0){
			return new SentimentResult("",0);
		}
	  //System.out.println(this.getMaxValue(myCategorizer.categorize(post)));
		//System.out.println(myCategorizer.getBestCategory(myCategorizer.categorize(post)));
		return new SentimentResult(myCategorizer.getBestCategory(myCategorizer.categorize(post)),
							this.getMaxValue(myCategorizer.categorize(post)));
	}
	
	public double getMaxValue(double[] values){
		double max = 0;
		for (int counter = 0; counter < values.length; counter++)		{
			if (Double.compare(values[counter], max) > 0){
					max = values[counter];
			}
		}  
		return (double) Math.round(max*1000)/1000;
	}
}