package sentimentanalyzer;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchType;
import org.elasticsearch.client.Client;
import org.elasticsearch.client.transport.TransportClient;
import org.elasticsearch.common.transport.InetSocketTransportAddress;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.common.xcontent.XContentBuilder;
import static org.elasticsearch.common.xcontent.XContentFactory.jsonBuilder;
import org.elasticsearch.search.SearchHit;

/**
 *  Třída pro připojení do elasticsearch a update dat o sentiment
 * 
 * @author Petr Škola
 */
public class ESConnect {
	private final Client client;
	private final SimpleDateFormat f = new SimpleDateFormat("d.M.yyyy HH:mm:ss");
		
	public ESConnect (){
		//Settings settings = ImmutableSettings.settingsBuilder().put("cluster.name", "sentiment").build(); //"elasticsearch"
		this.client = new TransportClient().addTransportAddress(new InetSocketTransportAddress("10.0.0.109",9300));
	}
	
	public void endSession(){
		this.client.close();
	}
	
  /**
   * Metoda načte všechna požadovaná data z elasticsearch a zjistí sentiment a uloží zpět do ES.
   * 
   * @param a Analyzer 
   */
	public void updateSentiment(Analyzer a){
    HashMap cat = new HashMap();
    cat.put("negative", -1);
    cat.put("neutral", 0);
    cat.put("positive", 1);
    
    SearchResponse scrollResp = this.client.prepareSearch("banky")
							.setSearchType(SearchType.SCAN)
							.setScroll(new TimeValue(30000))
							.setIndices("banky")
							.addFields("message")
							.setSize(100).execute().actionGet(); //100 hits per shard will be returned for each scroll
		//Scroll until no hits are returned
		long hits = scrollResp.getHits().totalHits();
		System.out.println("totalHits: " + hits);

		long i = 0;
		while (true) {
			try{
				for (SearchHit hit : scrollResp.getHits()) {
					i++;
					if(hit.fields().containsKey("message")){
						SentimentResult sent = a.detectSentiment(hit.field("message").getValue().toString());
						XContentBuilder json = jsonBuilder().startObject();
						json.field("sentiment", sent.resultClass);
						json.field("sentimentN", cat.get(sent.resultClass));
            json.field("sentimentWeight", sent.resultWeight);
						json.endObject();
						this.client.prepareUpdate("banky", hit.getType(), hit.getId())
										.setDoc(json)
										.execute().actionGet();
					}
					if (i%1000==0) System.out.println( f.format(new Date()) +"  " + i + "/" + hits );
				}
				scrollResp = this.client.prepareSearchScroll(scrollResp.getScrollId()).setScroll(new TimeValue(30000)).execute().actionGet();
				//Break condition: No hits are returned
				if (scrollResp.getHits().getHits().length == 0) {
					System.out.println( f.format(new Date()) +"  " + i + "/" + hits );
					break;
				}
			}catch(Exception ex){
				Logger.getLogger(ESConnect.class.getName()).log(Level.SEVERE, null, ex);
				break;
			}
		}
		System.out.print("\n");
	}
}