package sentimentanalyzer;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import opennlp.tools.doccat.DoccatModel;
import opennlp.tools.doccat.DocumentCategorizerME;
import opennlp.tools.doccat.DocumentSample;
import opennlp.tools.doccat.DocumentSampleStream;
import opennlp.tools.util.ObjectStream;
import opennlp.tools.util.PlainTextByLineStream;

/**
 *  Třída pro vytvoření trénovacího modelu podle zadaných trénovacích dat
 * 
 * @author Petr Škola
 */
public class Training {
	
  /**
   * Spustí proces trénování podle vstupních dat
   * 
   * @throws IOException 
   */
	public void train() throws IOException {
		String onlpModelPath = "model/";
		String trainingDataFilesPath = "trainingData/";
		String modelOutput = onlpModelPath + "model.bin";
		DoccatModel model = null;
		InputStream dataInputStream = null;
		BufferedOutputStream modelOut = new BufferedOutputStream(new FileOutputStream(modelOutput));
								
		List<File> files = listDirsInFolder(new File(trainingDataFilesPath));
		PrintWriter writer = new PrintWriter(trainingDataFilesPath + "/trainingData.txt", "UTF-8");
		
		for (File file : files) {
			String line;
			InputStream fis = new FileInputStream(file);
			InputStreamReader isr = new InputStreamReader(fis, "UTF-8");
			BufferedReader br = new BufferedReader(isr);
			while ((line = br.readLine()) != null) {
				if (line.length() > 0){
					writer.println(file.getName().replace(".txt", "") + " " + line);
				}
			}
		}
		writer.close();
		
		try {
		 // Read training data file
		 dataInputStream = new FileInputStream(new File(trainingDataFilesPath + "/trainingData.txt"));
		 // Read each training instance
		 ObjectStream<String> lineStream = new PlainTextByLineStream(dataInputStream, "UTF-8");
		 ObjectStream<DocumentSample> sampleStream = new DocumentSampleStream(lineStream);
		 // Calculate the training model
		 model = DocumentCategorizerME.train("cs", sampleStream, 20 , 300);
		 // Save model
		 model.serialize(modelOut);
		 
		} catch (IOException e) {
		 System.err.println(e.getMessage());
		} finally {
		 if (dataInputStream != null) {
			try {
			 dataInputStream.close();
			} catch (IOException e) {
			 System.err.println(e.getMessage());
			}
		 }
		}
	}
		
	private List<File> listDirsInFolder(File folder) {
		List<File> files = new ArrayList<File>();
    for (File fileEntry : folder.listFiles()) {
			if (fileEntry.isDirectory()) {
				files.addAll(listFilesInFolder(fileEntry));
			}
    }
		return files;
	}

	private List<File> listFilesInFolder(File folder) {
		List<File> files = new ArrayList<File>();
    for (File fileEntry : folder.listFiles()) {
			if (!fileEntry.isDirectory()) {
				File f = new File(fileEntry.getAbsolutePath());
				files.add(f);
			}
    }
		return files;
	}
}
