package fbconnector;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.elasticsearch.action.admin.indices.close.CloseIndexRequest;
import org.elasticsearch.action.admin.indices.exists.indices.IndicesExistsResponse;
import org.elasticsearch.action.admin.indices.mapping.put.PutMappingRequest;
import org.elasticsearch.action.admin.indices.open.OpenIndexRequest;
import org.elasticsearch.action.admin.indices.settings.put.UpdateSettingsRequest;
import org.elasticsearch.client.Client;
import org.elasticsearch.client.transport.TransportClient;
import org.elasticsearch.common.settings.ImmutableSettings;
import org.elasticsearch.common.transport.InetSocketTransportAddress;
import org.elasticsearch.node.Node;
import static org.elasticsearch.node.NodeBuilder.nodeBuilder;
import org.json.simple.JSONObject;

/**
 *
 * @author Petr Škola
 */
public class ESConnect {
	private Client client;
	private Node node = null;
	private String indexName;
		
	public ESConnect (String indexName, String ip, String clusterName){ 
    Client client;
    if (ip.length() > 0){
      if (clusterName.length() > 0){
        org.elasticsearch.common.settings.Settings settings = ImmutableSettings.settingsBuilder().put("cluster.name", clusterName).put("client.transport.sniff", true).build();
        client = new TransportClient(settings).addTransportAddress(new InetSocketTransportAddress(ip,9300));
      }else{
        client = new TransportClient().addTransportAddress(new InetSocketTransportAddress(ip,9300));
      }
      this.client = client;
    }else if (clusterName.length() > 0){
      node = nodeBuilder().client(true).clusterName(clusterName).node();
      this.client = node.client();
    }
		this.indexName = indexName;
	}

	public void postElasticSearch(JSONObject jsn) throws Exception{
		jsn.put("source", "Facebook");
    client.prepareIndex(this.indexName, jsn.get("type").toString(), jsn.get("id").toString())
                .setSource(jsn).execute().actionGet();		
	}
	
	public void postElasticSearch(List<JSONObject> jsonList) throws Exception{
		//if (this.isMapping == false) throw new Exception("Pro tento index neexistuje mapping");
		for (JSONObject jsnObj : jsonList){
			jsnObj.put("source", "Facebook");
			postElasticSearch(jsnObj);
		}
	}
	
	public boolean isIndex(){
		IndicesExistsResponse res = this.client.admin().indices().prepareExists(this.indexName).execute().actionGet();
		return res.isExists();
	}
	
	public void createIndex(){
		if (!isIndex()){
			client.admin().indices().prepareCreate(this.indexName).execute().actionGet();
		}
	}
		
	public void createMapping(String documentType, JSONObject mapping){
		try {
			PutMappingRequest pmr = new PutMappingRequest(this.indexName);
			pmr.source(mapping);
			pmr.type(documentType);
			pmr.ignoreConflicts(true);
			client.admin().indices().putMapping(pmr).actionGet();
		} catch (Exception e) {
			Logger.getLogger(ESConnect.class.getName()).log(Level.SEVERE, null, e);
		}
	}
	
	public void setIndexSettings(JSONObject analyzer){
		try {
			CloseIndexRequest cir = new CloseIndexRequest(this.indexName);
			client.admin().indices().close(cir).actionGet();
			
			UpdateSettingsRequest usrq = new UpdateSettingsRequest(this.indexName);
			usrq.settings(analyzer);
			client.admin().indices().updateSettings(usrq).actionGet();
			
			OpenIndexRequest oir = new OpenIndexRequest(this.indexName);
			client.admin().indices().open(oir);
		} catch (Exception e) {
			Logger.getLogger(ESConnect.class.getName()).log(Level.SEVERE, null, e);
		}
	}
	
	public void endSession(){
		this.client.close();   
    if (this.node != null){
      this.node.close();
    }
	}
}