package fbconnector;

import com.restfb.Connection;
import com.restfb.DefaultFacebookClient;
import com.restfb.types.Comment;
import com.restfb.types.Post;
import com.restfb.types.User;
import com.restfb.types.Page;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.elasticsearch.common.joda.time.DateTime;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

/**
 * Trida pro načtení dat z Facebookových profilů - posty, komentáře a uživatele
 *
 * @author Petr Škola
 */
class FBDownloader {

  private final com.restfb.FacebookClient facebookClient;
  private ESConnect ESconn;
  private ESConnect ESconnUsers;
  private ESConnect ESconnPage;
  private List<Object> FBPages;
  private final String COMMENTS = "comments";
  private final String USERS = "users";
  private Date minDate = null;

  /**
   *
   * @param accessToken
   */
  public FBDownloader(DefaultFacebookClient facebookClient) {
    this.facebookClient = facebookClient;
  }

  public void setESConnect(ESConnect ESconn) {
    this.ESconn = ESconn;
  }

  public void setESConnectUsers(ESConnect ESconn) {
    this.ESconnUsers = ESconn;
  }
  
  public void setESConnectPage(ESConnect ESconn) {
    this.ESconnPage = ESconn;
  }

  public void setFBPages(List<Object> FBPages) {
    this.FBPages = FBPages;
  }

  public void setMinDate(Date minDate) {
    this.minDate = minDate;
  }

  /**
   * Metoda pro započení stahování dat pro každou v configu definovanou stránku FB
   * 
   */
  public void startDownload() {
    for (Object page : FBPages) {
      String sPage = (String) page;
      System.out.println("Začínám stahovat: " + sPage);
      indexPageStats(sPage, ESconnPage);
      indexPosts(sPage, ESconn);
    }
  }

  /**
   * Metodat pro stažení postů a komentářů z FB zdi stránky
   * 
   * 
   * @param fbUser
   * @param ESconn 
   */
  private void indexPosts(String fbUser, ESConnect ESconn) {
    Connection<Post> myFeed = facebookClient.fetchConnection(fbUser + "/feed", Post.class);
    List<JSONObject> jsonList = new ArrayList<>();
    boolean stopDownload = false;
    SimpleDateFormat f = new SimpleDateFormat("d.M.yyyy HH:mm:ss");
    Set users = new HashSet();
    long i = 1;
    for (List<Post> feedItem : myFeed) {
      if (stopDownload) {
        break;
      }
      //if (i % 20 == 0 || i == 1) 
      DateTime start = new DateTime();
      //}
      int pN = 0;
      int pC = 0;
      long commentsDur = 0;
      long subCommentsDur = 0;

      for (Post post : feedItem) {
        if (this.minDate != null && post.getCreatedTime().compareTo(this.minDate) < 0) {
          stopDownload = true;
          break;
        }
        pN++;
        users.add(post.getFrom().getId());
        DateTime fbCommentsStart = new DateTime();
        Map r1 = getCommentFromPost(facebookClient, post.getId());
        List<Comment> comments = (List<Comment>) r1.get(this.COMMENTS);
        users.addAll((Collection) r1.get(this.USERS));
        commentsDur = commentsDur + (new DateTime().getMillis() - fbCommentsStart.getMillis()) / 1000;

        pC = pC + comments.size();

        jsonList.addAll(prepareJsonForIndex(post, fbUser));
        jsonList.addAll(prepareJsonForIndex(comments, post.getId(), null, fbUser));

        List<Comment> subComments;
        DateTime fbSubCommentsStart = new DateTime();
        for (Comment cmnt : comments) {
          users.add(cmnt.getFrom().getId());
          Map r2 = getCommentFromPost(facebookClient, cmnt.getId());
          subComments = (List<Comment>) r2.get(this.COMMENTS);
          users.addAll((Collection) r2.get(this.USERS));
          pC = pC + subComments.size();
          if (!subComments.isEmpty()) {
            jsonList.addAll(prepareJsonForIndex(subComments, post.getId(), cmnt.getId(), fbUser));
          }
        }
        subCommentsDur = subCommentsDur + (new DateTime().getMillis() - fbSubCommentsStart.getMillis()) / 1000;

        try {
          ESconn.postElasticSearch(jsonList);
        } catch (Exception ex) {
          Logger.getLogger(FBDownloader.class.getName()).log(Level.SEVERE, null, ex);
        }
      }
      System.out.println(f.format(new Date()) + ": Zpracována feed page " + i + " (počet postů: " + pN + ", počet komentářů: " + pC + ") za "
         + (new DateTime().getMillis() - start.getMillis()) / 1000 + " vteřin"); //, délka FB requestů: " + (commentsDur + subCommentsDur));
      i++;
    }
    indexUsers(users, fbUser);
  }

  /**
   * Získá komentáře pod postem
   *    
   * @param facebookClient
   * @param elementId
   * @return Map mapa uživatelů a komentářů
   */
  private Map getCommentFromPost(com.restfb.FacebookClient facebookClient, String elementId) {
    Map r = new HashMap();
    Set users = new HashSet();
    List<Comment> comments = new ArrayList();
    Connection<Comment> allComments = facebookClient.fetchConnection(elementId + "/comments", Comment.class);
    for (List<Comment> postComments : allComments) {
      for (Comment comment : postComments) {
        comments.add(comment);
        users.add(comment.getFrom().getId());
      }
    }
    r.put("comments", comments);
    r.put("users", users);
    return r;
  }

  /**
   * Přípraví Json pro post pro uložení do ES
   * 
   * @param post
   * @param page
   * @return List<JSONObject> pro uložení dat do Elasticsearch
   */
  private List<JSONObject> prepareJsonForIndex(Post post, String page) {
    List<JSONObject> jsonList = new ArrayList<>();
    Format f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    JSONObject lineJS = new JSONObject();
    String message = post.getMessage() == null ? post.getStory() : post.getMessage();
    
    lineJS.put("message", message);
    //lineJS.put("messageRaw",message.substring(0, message.length()>MAX_LEN_STRING_NA?MAX_LEN_STRING_NA:message.length()) );
    lineJS.put("userId", post.getFrom().getId());
    lineJS.put("userName", post.getFrom().getName());
    //lineJS.put("userName.raw", post.getFrom().getName());
    lineJS.put("created", f.format(post.getCreatedTime()));
    //lineJS.put("caption", post.getCaption());
    //lineJS.put("posttype", post.getType());
    lineJS.put("type", "post");
    lineJS.put("likes", getPostLikes(post.getId())); // post.getLikes() == null ? 0 : post.getLikes().getData().size());
    lineJS.put("page", page);
    //lineJS.put("page.raw", page);
    lineJS.put("level", -1);
    lineJS.put("id", post.getId());
    jsonList.add(lineJS);
    return jsonList;
  }

  /**
   * Přípraví Json pro data komentářů pro uložení do ES
   * 
   * @param comments
   * @param postId
   * @param parentCommentID
   * @param page
   * @return List<JSONObject>  pro uložení dat do Elasticsearch
   */
  private List<JSONObject> prepareJsonForIndex(List<Comment> comments, String postId, String parentCommentID, String page) {
    List<JSONObject> jsonList = new ArrayList<>();
    Format f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    if (comments.size() > 0) {
      for (Comment cmnt : comments) {
        JSONObject lnJS = new JSONObject();
        String message = cmnt.getMessage();
        lnJS.put("message", message);
        //lnJS.put("messageRaw", message.substring(0, message.length()>MAX_LEN_STRING_NA?MAX_LEN_STRING_NA:message.length()));
        lnJS.put("userId", cmnt.getFrom().getId());
        lnJS.put("userName", cmnt.getFrom().getName());
        //lnJS.put("userName.raw", cmnt.getFrom().getName());
        lnJS.put("created", f.format(cmnt.getCreatedTime()));
        lnJS.put("type", "comment");
        lnJS.put("likes", cmnt.getLikeCount());
        lnJS.put("postId", postId);
        lnJS.put("page", page);
        //lnJS.put("page.raw", page);
        lnJS.put("id", cmnt.getId());
        lnJS.put("level", parentCommentID == null ? 0 : 1);
        lnJS.put("parentId", parentCommentID == null ? postId : parentCommentID);

        jsonList.add(lnJS);
      }
    }
    return jsonList;
  }

  /**
   * Přípraví Json pro uživatelská data pro uložení do ES
   * 
   * @param user
   * @param page
   * @return JSONObject uživatele pro uložení do ES
   */
  private JSONObject prepareJsonForIndex(User user, String page) {
    Format f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    JSONObject lnJS = new JSONObject();
    lnJS.put("type", "user");
    lnJS.put("about", user.getAbout());
    lnJS.put("id", user.getId());
    lnJS.put("birthday", user.getBirthday());
    lnJS.put("gender", user.getGender());
    lnJS.put("homeTown", user.getHometownName());
    lnJS.put("locale", user.getLocale());
    lnJS.put("userType", user.getType());
    lnJS.put("userName", user.getUsername());
    lnJS.put("picture", user.getPicture());
    lnJS.put("page", page);
    //lnJS.put("education", user.getEducation().toString());

    return lnJS;
  }
  
  /**
   * Přípraví Json pro data o FB stránkce pro uložení do ES
   * 
   * @param pg
   * @param page
   * @return List<JSONObject> FB stránek pro uložení do ES
   */
  private List<JSONObject> prepareJsonForIndex(Page pg, String page) {
    List<JSONObject> jsonList = new ArrayList<>();
    Format f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    Format f2 = new SimpleDateFormat("yyyy-MM-dd");
    
    JSONObject lineJS = new JSONObject();
    lineJS.put("about", pg.getAbout());
    lineJS.put("category", pg.getCategory());
    lineJS.put("description", pg.getDescription());
    lineJS.put("link", pg.getLink());
    lineJS.put("created", f.format(new Date()));
    lineJS.put("type", "page");
    lineJS.put("likes", pg.getLikes());
    lineJS.put("page", page);
    lineJS.put("page.raw", page);
    lineJS.put("id", pg.getId()+"_"+f2.format(new Date()));
    jsonList.add(lineJS);
    return jsonList;
  }
  
  /**
   * Metoda vytvori index v ES
   *
   * @param index nazev indexu, ktery ma byt vytvoren
   */
  public JSONObject getAnalyzer(String index) {
    String analyzer = "{\"settings\": {\"analysis\": {\"filter\": {\"czech_stop\": {\"type\": \"stop\",\"stopwords\":  \"_czech_\"},\"czech_keywords\": {\"type\":       \"keyword_marker\",\"keywords\":   [\"x\"]}, \"czech_stemmer\": { \"type\":       \"stemmer\", \"language\":   \"czech\"}, \"syno_czech\":{\"type\":\"synonym\", \"synonyms_path\":\"synonym.txt\"}},\"analyzer\": {\"czech\": {\"tokenizer\":  \"standard\",\"filter\": [ \"lowercase\",\"czech_stop\", \"czech_keywords\", \"czech_stemmer\",\"syno_czech\"]}}}}}";
    JSONParser parser = new JSONParser();
    Object obj = null;
    try {
      obj = parser.parse(analyzer);
    } catch (ParseException ex) {
      Logger.getLogger(FBDownloader.class.getName()).log(Level.SEVERE, null, ex);
    }
    JSONObject jsonObject = (JSONObject) obj;
    return jsonObject;
  }

  /**
   * Metoda vytvori mapovani pro analyzer k indexu a typu, ktery je
   * specifikovan
   *
   * @param index mapovani, ktere ma byt vytvoren
   * @param typ k namapovani
   */
  public JSONObject prepareMapping(String typ) {
    JSONObject types = new JSONObject();
    
    JSONObject stringNotAnalyzed = new JSONObject();
    stringNotAnalyzed.put("index", "not_analyzed");
    stringNotAnalyzed.put("type", "string");

    JSONObject stringAnalyzed = new JSONObject();
    stringAnalyzed.put("index", "analyzed");
    stringAnalyzed.put("type", "string");
    
    JSONObject stringAnalyzedCZ = new JSONObject();
    stringAnalyzedCZ.put("index", "analyzed");
    stringAnalyzedCZ.put("analyzer", "czech");
    stringAnalyzedCZ.put("type", "string");
    
    JSONObject message = new JSONObject();
    types.put("message", stringAnalyzedCZ);
    
    JSONObject userId = new JSONObject();
    userId.put("type", "string");
    types.put("userId", userId);

    JSONObject userName = new JSONObject();
    types.put("userName", stringNotAnalyzed);

    JSONObject created = new JSONObject();
    created.put("format", "yyyy-MM-dd HH:mm:ss");
    created.put("type", "date");
    types.put("created", created);

    JSONObject postId = new JSONObject();
    postId.put("type", "string");
    postId.put("index" , "not_analyzed");
    types.put("postId", postId);

    JSONObject likes = new JSONObject();
    likes.put("type", "long");
    types.put("likes", likes);

    JSONObject page = new JSONObject();
    types.put("page", stringNotAnalyzed);

    JSONObject level = new JSONObject();
    level.put("type", "long");
    types.put("level", level);

    JSONObject sent = new JSONObject();
    sent.put("type", "string");
    sent.put("index", "not_analyzed");
    types.put("sentiment", sent);

    JSONObject sentWeight = new JSONObject();
    sentWeight.put("type", "long");
    types.put("sentimentWeight", sentWeight);	

    if (typ == "comment") {
      JSONObject parentId = new JSONObject();
      parentId.put("type", "string");
      types.put("parentId", parentId);
    }

    JSONObject mappingBody = new JSONObject();
    mappingBody.put("properties", types);

    return mappingBody;
  }

    /**
   * Metoda vytvori mapovani pro analyzer k indexu a typu, ktery je
   * specifikovan
   *
   * @param index mapovani, ktere ma byt vytvoren
   * @param typ k namapovani
   */
  public JSONObject preparePageMapping(String typ) {
    JSONObject types = new JSONObject();

    JSONObject about = new JSONObject();
    about.put("type", "string");
    types.put("about", about);

    JSONObject category = new JSONObject();
    category.put("type", "string");
    types.put("category", category);

    JSONObject description = new JSONObject();
    description.put("type", "string");
    types.put("description", description);

    JSONObject link = new JSONObject();
    link.put("type", "string");
    types.put("link", link);

    JSONObject created = new JSONObject();
    created.put("format", "yyyy-MM-dd HH:mm:ss");
    created.put("type", "date");
    types.put("created", created);

    JSONObject type = new JSONObject();
    type.put("type", "string");
    types.put("type", type);
    
    JSONObject likes = new JSONObject();
    likes.put("type", "long");
    types.put("likes", likes);

    JSONObject page = new JSONObject();
    page.put("type", "string");
    types.put("page", page);

    JSONObject pageRaw = new JSONObject();
    pageRaw.put("type", "string");
    pageRaw.put("index", "not_analyzed");
    types.put("page.raw", pageRaw);

    JSONObject id = new JSONObject();
    id.put("type", "string");
    types.put("id", id);
    
    JSONObject mappingBody = new JSONObject();
    mappingBody.put("properties", types);

    return mappingBody;
  }
  
  /**
   * provede uložení dat o uživatelích
   * 
   * @param users
   * @param FbUser 
   */
  private void indexUsers(Set users, String FbUser) {
    Iterator iter = users.iterator();
    while (iter.hasNext()) {
      String userId = (String) iter.next();
      try {
        User user = facebookClient.fetchObject(userId, User.class);
        ESconnUsers.postElasticSearch(prepareJsonForIndex(user, FbUser));
      } catch (Exception ex) {
        Logger.getLogger(FBDownloader.class.getName()).log(Level.SEVERE, null, ex);
      }
    }
  }
  
  /**
   * Provede uložení dat o FB stránkách 
   * 
   * @param fbPage
   * @param ESconnPage 
   */
  private void indexPageStats(String fbPage, ESConnect ESconnPage){
    Page pg = facebookClient.fetchObject(fbPage, Page.class);
    try {
      ESconnPage.postElasticSearch(prepareJsonForIndex(pg, fbPage));
    } catch (Exception ex) {
      Logger.getLogger(FBDownloader.class.getName()).log(Level.SEVERE, null, ex);
    }
  }
  
  /**
   * Získá přesný počet liků postu
   * 
   * @param postID
   * @return long počet liků
   */
  private long getPostLikes(String postID){
    Connection<Post.Likes> likes = facebookClient.fetchConnection(postID + "/likes", Post.Likes.class);
    long cnt = 0;
    for (List<Post.Likes> likePage : likes) {
      cnt = cnt + likePage.size();
    }
    return cnt;
  }
}
