package fbconnector;

import com.restfb.DefaultFacebookClient;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.commons.configuration.PropertiesConfiguration;

/**
 *
 * @author Petr Škola
 */
public class Main {
  /**
   * @param args the command line arguments
   */
  public static void main(String[] args) throws Exception {
    PropertiesConfiguration config = new PropertiesConfiguration();

    try {
      config.load(args[0]); // args[0]="./config.properties"  --  konektor-FB\config.properties
    } catch (Exception ex) {
      Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
      throw new Exception("Nepodařilo se načíst config");
    }
    
    MyConfigHandler mch = new MyConfigHandler(config);

    DefaultFacebookClient client = null;
    if (config.containsKey("appId") && config.containsKey("appSecret")) {
      //přihlášení pomocí aplikace na FB a její appId a app secret
      client = new FacebookClient(config.getString("appId"), config.getString("appSecret"));
    } else if (config.containsKey("accessToken")) {
      //přihlášení pomocí Access Token z Graph exploreru
      client = new FacebookClient(config.getString("accessToken"));
    } else {
      throw new Exception("Config neobsahuje přihlašovací údaje pro FB v \"appID\" a \"appSecret\" nebo \"accessToken\"");
    }

    FBDownloader fbDwnldr = new FBDownloader(client);
    
    String indexName = null;
    String indexNameUsers = null;
    String indexNamePage = null;
    if (config.containsKey("indexName")) {
      indexName = config.getString("indexName");
      indexNameUsers = config.getString("indexName") + "users";
      indexNamePage = config.getString("indexName") + "pages";
    } else {
      throw new Exception("Config neobsahuje název indexu v \"indexName\"");
    }
    
    //připojení do ES pomocí Java API
    ESConnect escon = new ESConnect(indexName, 
                                    mch.getStringConfigValue("esIp", "", false), 
                                    mch.getStringConfigValue("clusterName", "", false));
    ESConnect esconUsers = new ESConnect(indexNameUsers,
                                         mch.getStringConfigValue("esIp", "", false), 
                                         mch.getStringConfigValue("clusterName", "", false));
    ESConnect esconPage = new ESConnect(indexNamePage,
                                         mch.getStringConfigValue("esIp", "", false), 
                                         mch.getStringConfigValue("clusterName", "", false));
        
    //příprava indexu 
    if (!escon.isIndex()) {
      escon.createIndex();
    }
    if (!esconUsers.isIndex()) {
      esconUsers.createIndex();
    }
    if (!esconPage.isIndex()) {
      esconPage.createIndex();
    }
    escon.createMapping("post", fbDwnldr.prepareMapping("post"));
    escon.createMapping("comment", fbDwnldr.prepareMapping("comment"));
    //TODO: esconUser.createMapping("user", fbDwnldr.prepareMapping("user"));
    esconPage.createMapping("page", fbDwnldr.prepareMapping("page"));
    escon.setIndexSettings(fbDwnldr.getAnalyzer(indexName));

    List<Object> pages = null;
    if (config.containsKey("fbPage")) {
      pages = config.getList("fbPage");
    } else {
      throw new Exception("Config neobsahuje FB stránky ke stažení v \"fbPage\"");
    }

    Date minDate = null;
    SimpleDateFormat f = new SimpleDateFormat("d.M.yyyy HH:mm:ss");
    if (config.containsKey("minDate")) {
      minDate = f.parse(config.getString("minDate"));
    } else {
      minDate = f.parse("1.1.2014 00:00:00");
    }

    //načítání dat z FB a odesílání do ES	
    fbDwnldr.setESConnect(escon);
    fbDwnldr.setFBPages(pages);
    fbDwnldr.setESConnectUsers(esconUsers);
    fbDwnldr.setESConnectPage(esconPage);
    fbDwnldr.setMinDate(minDate);

    fbDwnldr.startDownload();

    escon.endSession();
    esconUsers.endSession();
    esconPage.endSession();
    System.out.println("Dokončeno");
  } 
}
